<?php
//
//  Nagios Core Config Manager
//  Copyright (c) 2010-2014 Nagios Enterprises, LLC
//
//  File: index.php
//  Desc: First page of the CCM. This page is designed to create either a Nagios XI compatable
//        page using the XI do_page_start or a Nagios CCM standalone page.
//

// Define environment
if (file_exists(dirname(__FILE__).'/../../../config.inc.php')) {
    define('ENVIRONMENT', 'nagiosxi');
} else {
    define('ENVIRONMENT', 'nagioscore');
}

// Start the session and initialization based on environment
if (ENVIRONMENT == "nagiosxi") {

    // Include the Nagios XI helper functions through the component helper file and initialize
    // anything we will need to authenticate ourselves to the CCM
    require_once(dirname(__FILE__).'/../componenthelper.inc.php');
    pre_init();
    init_session();

    // Check if automatic integretion exists
    $separate_ccm_login = get_option("separate_ccm_login", 0);

} else {
    ob_start();
    session_start();
}

// Set the location of the CCM root directory
define('BASEDIR', dirname(__FILE__).'/');
require_once('includes/session.inc.php');
?>
<?php
// Display page heading
if (ENVIRONMENT == "nagiosxi") {
    do_page_start(array("page_title" => 'Nagios Core Config Manager'), true);
} else {
?>
<!DOCTYPE html>
<html lang="en">
<head>
    <title>Nagios Core Config Manager</title>
    <link rel='stylesheet' type='text/css' href='css/style_core.css?<?php echo VERSION; ?>' />
    <script type="text/javascript" src="javascript/jquery-1.7.2.min.js?<?php echo VERSION; ?>"></script>
    <script type="text/javascript" src="javascript/main_js.js?<?php echo VERSION; ?>"></script>
    <script type="text/javascript">
    var NAGIOSXI=<?php if (ENVIRONMENT == 'nagiosxi') { echo "true"; } else { echo "false"; } ?>;
    </script>
    <style type="text/css">
    #contentWrapper { margin: 0px auto; width: 80%; float: left; }
    </style>
</head>
<body>
<?php
}

// Let's display the red asterik next to apply config if we have apply configuration needed
if (ENVIRONMENT == "nagiosxi") {
    $ac_needed = get_option("ccm_apply_config_needed", 0);
    if ($ac_needed == 1) {
?>
<script type="text/javascript">
    $(document).ready(function() {
        window.parent.$("#ccm-apply-menu-link").html('<span class="tooltip-apply" title="<?php echo gettext("There are modifications to objects that have not been applied yet. Apply configuration for new changes to take affect."); ?>"><i class="fa fa-asterisk urgent"></i> <?php echo gettext("Apply Configuration"); ?></span>');
        window.parent.$('.tooltip-apply').tooltip({ tooltipClass: 'ccm-tooltip' });
    });
</script>
<?php
    }
}
?>
    <div id="screen-overlay"></div>
    <div id="loginMsgDiv" <?php if (ENVIRONMENT == "nagiosxi" && $separate_ccm_login == 0) { echo 'style="display: none;"'; } ?>>
        <span <?php if(!($_SESSION['loginStatus'] === false)) echo "class='deselect'"; ?>>
            <div <?php if($_SESSION['loginStatus'] === false) echo "class='error'"; ?>>
                <?php if (!empty($_SESSION['loginMessage'])) { echo $_SESSION['loginMessage']; } ?>
            </div>
        </span>
    </div>
<?php
// Display the actual page through the page router...
print page_router();

// Create page ending based on environment
if (ENVIRONMENT == "nagiosxi") {
    do_page_end(true);
} else {
?>
</body>
</html>
<?php
}
ob_end_flush();
?>